#include "chil_global.h"

#define CLEAR_PIXEL(igate) \
  for (j=0;j<ray->ntotal;j++) *(ray->data[j]+igate) = chil_no_signal_value
#define CLEAR_FIELD_PIXEL(ifield, igate) \
  *(ray->data[ifield]+igate) = chil_no_signal_value



int
CHILSubtractNoisePolar(Ray *ray, float nezh, float nezv, float nezhv,
		       float threshold)
{
  int i, j, iZh, iZdr, iLdr;
  float range2;
  float PZh, PZv, PZhv;
  float Pnezh, Pnezv, Pnezhv;

  if (!ray->range_normalised || (iZh = CHILFindField(ray, ZH)) < 0 ) { 
    return CHIL_NOTDONE;
  }

  iZdr = CHILFindField(ray, ZDR);
  iLdr = CHILFindField(ray, LDR);

  if (iZdr < 0 && iLdr < 0) {
    Pnezh = pow(10,0.1*nezh);
    for (i=0;i<ray->ngates;i++) {
      range2=(ray->range[i]*ray->range[i]);
      PZh=(pow(10,0.1* (*(ray->data[iZh]+i))))/range2;
      if (PZh <= Pnezh*threshold) {
	CLEAR_PIXEL(i);
      }
      else {
	PZh -= Pnezh;
	*(ray->data[iZh]+i) = 10.0 * log10(PZh*range2);
      }
    }
    return CHIL_OK;
  }

  Pnezh = pow(10,0.1*nezh);
  Pnezv = pow(10,0.1*nezv);
  Pnezhv = pow(10,0.1*nezhv);

  for (i=0;i<ray->ngates;i++) {
    range2=(ray->range[i]*ray->range[i]);
    PZh=(pow(10,0.1* (*(ray->data[iZh]+i))))/range2;
    if (PZh <= Pnezh*threshold) {
      CLEAR_PIXEL(i);
    }
    else {
      PZh -= Pnezh;
      if (iZdr >= 0) {
	PZv=(pow(10,0.1* (*(ray->data[iZh]+i)-*(ray->data[iZdr]+i))))/range2;
	if (PZv <= Pnezv*threshold) {
	  CLEAR_FIELD_PIXEL(iZdr, i);
	}
	else {
	  PZv -= Pnezv;
	  *(ray->data[iZdr]+i) = 10.0 * log10(PZh/PZv);
	}
      }
      if (iLdr >= 0) {
	PZhv=(pow(10,0.1* (*(ray->data[iZh]+i)+*(ray->data[iLdr]+i))))/range2;
	if (PZhv <= Pnezhv*threshold) {
	  CLEAR_FIELD_PIXEL(iLdr, i);
	}
	else {
	  PZhv -= Pnezhv;
	  *(ray->data[iLdr]+i) = 10.0 * log10(PZhv/PZh);
	}
      }
      *(ray->data[iZh]+i) = 10.0 * log10(PZh*range2);
    }
  }
  return CHIL_OK;
}

int
CHILSuppressClutterWithLdr(Ray *ray, float maxldr)
{
  int i, j, iLdr;
  if ((iLdr = CHILFindField(ray, LDR)) < 0) {
    return CHIL_NOTFOUND;
  }

  for (i=0;i<ray->ngates;i++) {
    if (*(ray->data[iLdr]+i) > maxldr && *(ray->data[iLdr]+i) 
	!= chil_no_signal_value) {
      CLEAR_PIXEL(i);
    }
  }
  return CHIL_OK;
}
